<?php

namespace Drupal\abtestui\Controller;

use Drupal\abtestui\Service\TestStorage;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ABTestUIDefaultController.
 *
 * @package Drupal\abtestui\Controller
 */
class ABTestUIDefaultController extends ControllerBase {

  /**
   * THe custom abtestui test storage.
   *
   * @var \Drupal\abtestui\Service\TestStorage
   */
  private $testStorage;

  private $renderer;

  private $dateFormatter;

  private $analyticsConfig;

  private $abtestuiPath;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('abtestui.test_storage'),
      $container->get('renderer'),
      $container->get('date.formatter'),
      $container->get('config.factory')
    );
  }

  public function __construct(
    TestStorage $testStorage,
    RendererInterface $renderer,
    DateFormatterInterface $dateFormatter,
    ConfigFactoryInterface $configFactory
  ) {
    $this->testStorage = $testStorage;
    $this->renderer = $renderer;
    $this->dateFormatter = $dateFormatter;
    $this->analyticsConfig = $configFactory->get('google_analytics.settings');
    $this->abtestuiPath = drupal_get_path('module', 'abtestui');
  }

  /**
   * Return the list page.
   *
   * @return array
   *   Render array.
   */
  public function testList() {
    $tests = $this->testStorage->loadMultiple();
    $alphabets = range('A', 'Z');

    $output = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'test-content',
        ],
      ],
    ];
    $header = [
      t('ID'),
      t('Test'),
      t('Created'),
      t('Status'),
    ];
    $rows = [];
    $active_array = [t('Inactive'), t('Active')];

    foreach ($tests as $test) {
      $test_data = [
        '#type' => 'container',
        '#attributes' => [
          'class' => [
            'test-name',
            'test-name-' . $test['tid'],
          ],
        ],
        'test_name_col' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => [
              'test-name-col',
            ],
          ],
          'test_name' => [
            '#title' => $test['name'],
            '#type' => 'link',
            '#url' => Url::fromRoute('abtestui.test_edit_form', ['ab_test_id' => $test['tid']]),
          ],
        ],
        'variations_col' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => [
              'variations-col',
            ],
          ],
          'base_url' => [
            '#type' => 'container',
            '#attributes' => [
              'class' => [
                'base-url',
                'ab-variations',
              ],
            ],
            'list_counter' => [
              '#type' => 'container',
              '#attributes' => [
                'class' => [
                  'list-counter',
                ],
              ],
              'markup' => [
                '#markup' => '(' . $alphabets[0] . ')',
              ],
            ],
            'variation_data' => [
              '#type' => 'container',
              '#attributes' => [
                'class' => [
                  'variation-data',
                ],
              ],
              'variation_name' => [
                '#type' => 'container',
                '#attributes' => [
                  'class' => [
                    'variation-name',
                  ],
                ],
                'markup' => [
                  '#markup' => t('Base URL [@odd%]:', ['@odd' => $test['base_odd']]) . '<a href="/' . $this->abtestuiPath . '/help/base-url.html" class="help-modal"><img width="20" height="20" src="/' . $this->abtestuiPath . '/img/help.svg"></a>',
                ],
              ],
              'variation_url' => [
                '#type' => 'container',
                '#attributes' => [
                  'class' => [
                    'variation-url',
                  ],
                ],
                'link' => [
                  '#title' => $test['base_url'],
                  '#type' => 'link',
                  '#url' => Url::fromUri($test['base_url']),
                ],
              ],
            ],
          ],
        ],
      ];

      if ($test['analytics_url']) {
        $test_data['test_name_col']['analitycs_url'] = [
          '#title'  => t('Analytics Results'),
          '#type'   => 'link',
          '#url'    => Url::fromUri($test['analytics_url']),
          '#prefix' => '<br>',
        ];
      }

      $i = 1;
      /** @var array $variation */
      foreach ($test['variations'] as $variation) {
        $test_data['variations_col']['variation_' . $i] = [
          '#type' => 'container',
          '#attributes' => [
            'class' => [
              'ab-variations',
            ],
          ],
          'list_counter' => [
            '#type' => 'container',
            '#attributes' => [
              'class' => [
                'list-counter',
              ],
            ],
            'markup' => [
              '#markup' => "($alphabets[$i])",
            ],
          ],
          'variation_data' => [
            '#type' => 'container',
            '#attributes' => [
              'class' => [
                'variation-data',
              ],
            ],
            'variation_name' => [
              '#type' => 'container',
              '#attributes' => [
                'class' => [
                  'variation-name',
                ],
              ],
              'markup' => [
                '#markup' => t('@variation_name [@odd%]:', ['@variation_name' => $variation['name'], '@odd' => $variation['odd']]),
              ],
            ],
            'variation_url' => [
              '#type' => 'container',
              '#attributes' => [
                'class' => [
                  'variation-url',
                ],
              ],
              'link' => [
                '#title' => $variation['url'],
                '#type' => 'link',
                '#url' => Url::fromUri($variation['url']),
              ],
            ],
          ],
        ];
        $i++;
      }

      $rows[] = [
        $test['tid'],
        $this->renderer->render($test_data),
        $this->dateFormatter->format($test['created'], 'short'),
        $active_array[(int) $test['active']],
      ];
    }

    if (empty($this->analyticsConfig->get('account'))) {
      $ga_account_id = '<a href="' . Url::fromRoute('google_analytics.admin_settings_form')->toString() . '">' . $this->t('Set up') . '</a>';
    }
    else {
      $ga_account_id = '#' . $this->analyticsConfig->get('account');
    }

    $output['info_bar'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'info-bar',
        ],
      ],
      'ga_account' => [
        // @todo: Maybe add a link to the GA settings page, if account is NULL.
        '#markup' => ('<span class="ga-info"><span class="label">'
          . t('Google Analytics ID:')
          . ' </span><span class="ga-value">' . $ga_account_id . '</span></span>'),
      ],
      'ga_check_results' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => [
            'ga-check-results',
          ],
        ],
        'link' => [
          '#title' => t('How to check the results in Google Analytics'),
          '#type' => 'link',
          '#url' => Url::fromUri('base:/' . $this->abtestuiPath . '/help/check-ga-results.html'),
          '#options' => [
            'attributes' => [
              'class' => [
                'help-modal',
              ],
            ],
          ],
        ],
      ],
      'ab_test_tool_help' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => [
            'ab-test-tool-help',
          ],
        ],
        'link' => [
          '#title' => t('How does this A/B Test tool work?'),
          '#type' => 'link',
          '#url' => Url::fromUri('base:/' . $this->abtestuiPath . '/help/ab-test-tool-help.html'),
          '#options' => [
            'attributes' => [
              'class' => [
                'help-modal',
              ],
            ],
          ],
        ],
      ],
    ];

    $output['add'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'clearfix',
          'action-bar',
          'first',
        ],
      ],
      'link' => [
        '#title' => t('+ Add new test'),
        '#type' => 'link',
        '#url' => Url::fromRoute('abtestui.test_add_form'),
        '#options' => [
          'attributes' => [
            'class' => [
              'add-test-link',
              'first',
            ],
          ],
        ],
      ],
    ];

    $output['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => [
        'class' => [
          'ab-test-table',
        ],
      ],
    ];

    $output['add_bottom'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'clearfix',
          'action-bar',
          'last',
        ],
      ],
      'link' => [
        '#title' => t('+ Add new test'),
        '#type' => 'link',
        '#url' => Url::fromRoute('abtestui.test_add_form'),
        '#options' => [
          'attributes' => [
            'class' => [
              'add-test-link',
              'last',
            ],
          ],
        ],
      ],
    ];

    // Add library.
    $output['#attached']['library'][] = 'abtestui/admin_design';

    return $output;
  }

}
